body {
	background-color: #cccccc;
	font-family: Verdana, Lucida, Arial;
	font-size: 10px;
	margin-left: 10px;
	margin-top: 10px;
	text-align: center;
}
td {
	font-size: 10px;
}
table {
	margin-left: auto;
	margin-right: auto;
}
#menu {
	font-size: 11px;
	text-align: left;
	height: 30px;
}
#directory {
	background-color: #ffffff;
	border: 1px solid #666666;
	padding: 20px;
	text-align: left;
	top: 0px;
}
#folder {
	float: left;
	height: 100px;
	padding: 2px;
	text-align: center;
	width: 120px;
}
#picture img {
	border: 1px solid #000000;
}
a {
	color: #5555aa;
}
a img {
	border: 2px solid #5555aa;
}
a:hover {
	color: #ff9400;
}
a:hover img {
	border: 2px solid #ff9400;
}
#folder img {
	border: 0px;
}
.info {
	background-color: #eeeeee;
	border: 1px dashed #888888;
	color: #666666;
	font-size: 10px;
	margin: 10px;
	padding: 5px;
}
