<div>
  <div id="menu">
$MENU
  </div>
  <table>
    <tr>
      <td id="directory">
        <div>
$FILES
        </div>
      </td>
    </tr>
    $DIRCOMMENT
		<tr>
			<td>
    		Indexed by <a href="http://apachegallery.dk">Apache::Gallery</a> - Copyright &copy; 2001-2003 Michael Legart - <a href="http://www.hestdesign.com/">Hest Design!</a>
			</td>
		</tr>
  </table>
</div>
