<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
	"http://www.w3.org/TR/html4/strict.dtd"> 

<html>
<head>
	<title>$TITLE</title>
	<link rel="stylesheet" href="/gallery.css">
	$META
</head>

<body>
$MAIN
</body>
</html>
