<td width="60%">
          <center>
            <table align="center" class="info">
              <tr>
                <td>
									$COMMENT
									$INFO
                </td>
              </tr>
            </table>
          </center>
</td>
